use Data::Dumper;
##################################################################################
#
#	Armagetron Server Stats XML Parser 
#	Programmed By: .dBd|Zoid
#
#	This is a simple XMl parser dedicated to converting armagetron
#	server stats from http://wrtlprnft.ath.cx/serverlist/serverxml.php
#	into an HTML formatted output.  It includes color codes but does not
#	put white backgrounds on dark colors like arma does.
#	This is totally free open source so do whatever you want with this,
#	not that there's much else that can be done other than polishing.
#	Use:
#		you'll need a batch or cron job to pull down server stats
#		from the URL listed above.  Here's an example:
#
#-----------------------------------------------------------------------------------
#
#	#!/bin/bash
#		while [ 1 ]
#		do
#			if `wget -N http://wrtlprnft.ath.cx/serverlist/serverxml.php`
#			then
#				mv serverxml.php serverxml.xml
#				echo "got it"
#				perl armaSXMLParser.pl
#			else
#				sleep 20
#			fi
#			sleep 20
#		done
#
#-----------------------------------------------------------------------------------
#
#	the above script will pull down server stats into serverxml.xml every
#	twenty seconds from wrtlprnft's server and run this script each time
#	the file is received properly.  
#	This script acts directly on serverxml.xml unless you change it below.
##################################################################################



##	GLOBAL STATS		##
##	DATE AND TIME STUFF	##
my $ampm = "am";
my ($sec,$min,$hr,$day,$month,$ryear) = localtime(time);
$year = $ryear + 1900;
$month++;
if ( $hr > 12 ) { $hr = $hr - 12; $ampm = "pm"; }
my $datetime = "$hr:$min $ampm $month/$day/$year";

my $players_online = 0;		#number of players online total

sub replace_colors {
	#replaces 0xrrggbb with HTML font color tag
	#color codes with improper formaqt are displayed as normal text
	my $in = @_[0];
	$in =~ s/0x([a-f,A-F,0-9]{6})/<font color=\"\#$1">/g;
	return $in;
}



##	LOAD XML FILE AND CLOSE		##
my $chunk = "";
open(FOO,"<serverxml.xml"); #readonly mode please!
foreach my $line (<FOO>) {
	$chunk .= $line;
}
close(FOO);
#close and let the fiie alone so we don't run into access issues


##	HACK'ISH LF REPLACEMENT SO WE CAN DO LARGE-CHUNK REGEX		##
$chunk =~ s/\n/\[linefeed\]/g;
#	note: this may introduce bugs if a xml contains [linefeed] 
#	if that's the case, just change this to something less obvious...
#	I had some issues getting special characters into regex statements
#	but using a relatively inert control code is best


##	EXTRACT ALL DATA BETWEEN <Servers> TAGS		##
my $servers_chunk;
if ( $chunk =~ /\<Servers.*?\>(.*)?\<\/Servers\>/g ) { $servers_chunk = $1; }

##	SPLIT THE CHUNK INTO INFO ABOUT EACH SERVER	##
my @servers;	#an array of info for each server
while ( $servers_chunk =~ /<Server(.*?)>(.*?)<\/Server>/g )  {
	my @playerslist; #temporary array to hold list of players, this is joined later
	my %server;	 #hash of info for this server
	my $server_info = $1;	#server info parameters from within the <Server> tag
	my $playerschunk = $2;	#specifically: only looks for <Players> tags within Server block
	my @si = split(/\[linefeed\]/,$server_info);	#only want to operate on individual lines now
	##	PULL OUT THE SERVER INFO	##
	foreach my $sopt ( @si ) {
		my $option;	#option is like "name" or "ip" or "description"
		my $value;	#value is the value of the option
		#split each line into option="value" pairs
		if ( $sopt =~ /(.*?)=\"(.*?)\"/ ) { $option = $1; $value = $2; }
		$option =~ s/ //g;	#remove spaces from options (specifically: whitspaces acting as tabs)
		$option =~ s/\t//g;	#remove tabs from options 
		$value =~ s/^\"//g;	#chop off quotes at beginning of values
		$value =~ s/\"$//g;	#chop of quotes at end of values
		$server{$option} = $value;	#store
	}	
	##	PULL OUT PLAYER LIST	##
	while ( $playerschunk =~ /<Player name\=\"(.*?)\".*?\/\>/g ) {	#for each player tag with a player name
		my $pl = $1;
		### highlight dbd players ###
		#omit this unless it's for dbd forums plz
		if ($pl =~ m/\.dbd\|/i ) { $pl = "<span class='dbd'>$pl</span>"; }
		### done highlight dbd players ###
		push @playerslist,$pl;	#push the player name onto our temp array
		$players_online++;
	}
	##	STORE EVERYTHING	##
	push @servers,{info=>{%server}, players=>join(" , ",@playerslist)};
	##	GET COUNT OF PLAYERS ONLINE	##
	#player names are joined and delimited by " , " (no quotes)
	#change this to whatever delimiter you want for display purposes	
}
##	REFORMAT	##
# 	i.e. just pull out only the info we want from the big array of hashes above
#	modify this by just adding whatever extra info you want
#	all hash keys are equal to the name of the "option" (like name, ip, description)
my %server_list;	#the big one! this will have everything we want to display
foreach my $server_stuff ( @servers ) {		#go through each server we've extracted
	my $stripname = $server_stuff->{'info'}->{'name'};	#make a special alphanumeric name for key
	$stripname =~ s/0x[0-9,a-f,A-F]{6}//g;	#remove color codes from the alphanumeric key
	$server_list{ $stripname } = {	#store the info we want into this nice new hash
		name => replace_colors($server_stuff->{'info'}->{'name'}),	#server name with color and stuff
		ip => $server_stuff->{'info'}->{'ip'},				#ip addy
		numplayers => $server_stuff->{'info'}->{'numplayers'},		#number of players online
		version => $server_stuff->{'info'}->{'version'},		#version of server software
		description => replace_colors($server_stuff->{'info'}->{'description'}),	#description
		url => replace_colors($server_stuff->{'info'}->{'url'}),	#url of server
		players => replace_colors($server_stuff->{'players'})		#list of players as a single string
	};
}

##	MAKE HTML OUTPUT	##
#	output is initially stored in $outchunk so we don't
#	have to keep the file open while working.

# HTML header stuff
my $outchunk = "<html>";
$outchunk .= "
<style>
.servername {
	font-family: arial,helvetica;
	font-size: 12px;
	color: #00ff00;
}
.others {
	font-family: arial,helvetica;
	font-size: 10px;
	color: #00ff00;
}
.dbd {
	font-size: 12px;
	font-weight: bold;
	color: #ffff00;
}
.label {
	font-family: arial,helvetica;
	font-size: 11px;
	color: #88ff88;
}
.empty {
	font-family: arial,helvetica;
	font-size: 11px;
	color: #FF5555;
}
.whitebg {
	background: #ffffff;
}
.updated {
	font-family: arial,helvetica;
	font-size: 12px;
	color: #9999ff;
}

</style>
<body bgcolor='#000000'>

<span class='updated'>
Last Updated: $datetime<br>
Total players online: " . $players_online . " <br>
</span>
<br><br>

<table border=0 cellspacing=0 cellpadding=0>
";

#now go trough each server keyed by alphanumeric name and spit out our info
foreach my $server( keys %server_list ) {
	if ( $server_list{$server}->{'players'} ) {

	$outchunk .= "<tr><td height=10></td></tr>";
	$outchunk .= "<tr><td class='servername'>";
	$outchunk .= 	"<div class='servername'>" . 
			$server_list{$server}->{'name'} . 
			"</div>";
	$outchunk .= "</td></tr><tr><td height=5></td></tr><tr><td class='others'>";
	$outchunk .= "<table border=0 cellspacing=0 cellpadding=0><tr><td width=50px>&nbsp;</td>"; 
	$outchunk .= "<td class='others' width=200>";
	$outchunk .= "<span class='label'>Description: </span>";
	$outchunk .= $server_list{$server}->{'description'};
	$outchunk .= "</td></tr></table>";
	$outchunk .= "</td></tr>";

		$outchunk .= "<tr><td height=5></td></tr><tr><td>";
		$outchunk .= "<table border=0 cellspacing=0 cellpadding=0><tr><td width=50px>&nbsp;</td>"; 
		$outchunk .= "<td width=200>";
		$outchunk .= "<span class='label'>Online: </span>";
		$outchunk .= "<span class='others'>" . $server_list{$server}->{'players'} . "</span>";
		$outchunk .= "</td></tr></table>";
		$outchunk .= "</td></tr>";
#	} else {
#		$outchunk .= "<tr><td height=5></td></tr><tr><td>";
#		$outchunk .= "<table border=0 cellspacing=0 cellpadding=0><tr><td width=50px>&nbsp;</td>"; 
#		$outchunk .= "<td class='others' width=200>";
#		$outchunk .= "<span class='label'>Online: </span>";
#		$outchunk .= "<span class='empty'>Server is Empty</span>";
#		$outchunk .= "</td></tr></table>";
#		$outchunk .= "</td></tr>";
	
	}
}

#HTML footers
$outchunk .= "</table></body></html>";

##	OUTPUT THE HTML		##
open (OUT,">OUT.html");
print OUT $outchunk;
close(OUT);












